# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.2

import enum

import winrt.system
from winrt import _winrt_windows_devices_adc_provider

__all__ = [
    "ProviderAdcChannelMode",
    "IAdcControllerProvider",
    "IAdcProvider",
]

class ProviderAdcChannelMode(enum.IntEnum):
    SINGLE_ENDED = 0
    DIFFERENTIAL = 1

IAdcControllerProvider = _winrt_windows_devices_adc_provider.IAdcControllerProvider
IAdcProvider = _winrt_windows_devices_adc_provider.IAdcProvider
